@@ -1,3 +1,5 @@
+import contextlib
+
 from sqlalchemy import BLOB
 from sqlalchemy import BOOLEAN
 from sqlalchemy import Boolean
@@ -25,6 +27,7 @@ from sqlalchemy import Index
 from sqlalchemy import INT
 from sqlalchemy import Integer
 from sqlalchemy import Interval
+from sqlalchemy import JSON
 from sqlalchemy import LargeBinary
 from sqlalchemy import literal
 from sqlalchemy import MetaData
@@ -35,6 +38,7 @@ from sqlalchemy import NVARCHAR
 from sqlalchemy import PrimaryKeyConstraint
 from sqlalchemy import schema
 from sqlalchemy import select
+from sqlalchemy import Sequence
 from sqlalchemy import SmallInteger
 from sqlalchemy import sql
 from sqlalchemy import String
@@ -53,14 +57,22 @@ from sqlalchemy import VARCHAR
 from sqlalchemy.dialects.mysql import base as mysql
 from sqlalchemy.dialects.mysql import insert
 from sqlalchemy.dialects.mysql import match
+from sqlalchemy.orm import DeclarativeBase
+from sqlalchemy.orm import Mapped
+from sqlalchemy.orm import mapped_column
 from sqlalchemy.sql import column
+from sqlalchemy.sql import delete
 from sqlalchemy.sql import table
+from sqlalchemy.sql import update
+from sqlalchemy.sql.ddl import CreateSequence
 from sqlalchemy.sql.expression import bindparam
 from sqlalchemy.sql.expression import literal_column
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import AssertsCompiledSQL
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import eq_ignore_whitespace
+from sqlalchemy.testing import expect_deprecated
+from sqlalchemy.testing import expect_raises
 from sqlalchemy.testing import expect_warnings
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import mock
@@ -173,16 +185,45 @@ class CompileTest(ReservedWordFixture, fixtures.TestBase, AssertsCompiledSQL):
             schema.CreateIndex(idx), "CREATE INDEX test_idx1 ON testtbl (data)"
         )
 
-    def test_create_index_with_prefix(self):
+    def test_create_index_delegates_to_dialect_option(
+        self, thirdparty_dialect
+    ):
+        """test the original 3rd party dialect issue in #13134, which is
+        that visit_create_index() doesn't assume kw like "length" are
+        present"""
+
+        @thirdparty_dialect
+        class ThirdPartyDialect(mysql.MySQLDialect):
+            name = "thirdparty"
+            construct_arguments = []
+
+        m = MetaData()
+        tbl = Table("testtbl", m, Column("data", String(255)))
+        idx = Index("test_idx1", tbl.c.data)
+
+        self.assert_compile(
+            schema.CreateIndex(idx),
+            "CREATE INDEX test_idx1 ON testtbl (data)",
+            dialect=ThirdPartyDialect(),
+        )
+
+    @testing.combinations("mysql", "mariadb", argnames="dialect_name")
+    def test_create_index_with_prefix(self, dialect_name):
         m = MetaData()
         tbl = Table("testtbl", m, Column("data", String(255)))
         idx = Index(
-            "test_idx1", tbl.c.data, mysql_length=10, mysql_prefix="FULLTEXT"
+            "test_idx1",
+            tbl.c.data,
+            **{
+                f"{dialect_name}_length": 10,
+                f"{dialect_name}_prefix": "FULLTEXT",
+            },
         )
 
         self.assert_compile(
             schema.CreateIndex(idx),
             "CREATE FULLTEXT INDEX test_idx1 ON testtbl (data(10))",
+            dialect=dialect_name,
         )
 
     def test_create_index_with_text(self):
@@ -224,36 +265,43 @@ class CompileTest(ReservedWordFixture, fixtures.TestBase, AssertsCompiledSQL):
             "CREATE INDEX test_idx1 ON testtbl (created_at desc)",
         )
 
-    def test_create_index_with_parser(self):
+    @testing.combinations("mysql", "mariadb", argnames="dialect_name")
+    def test_create_index_with_parser(self, dialect_name):
         m = MetaData()
         tbl = Table("testtbl", m, Column("data", String(255)))
         idx = Index(
             "test_idx1",
             tbl.c.data,
-            mysql_length=10,
-            mysql_prefix="FULLTEXT",
-            mysql_with_parser="ngram",
+            **{
+                f"{dialect_name}_length": 10,
+                f"{dialect_name}_prefix": "FULLTEXT",
+                f"{dialect_name}_with_parser": "ngram",
+            },
         )
 
         self.assert_compile(
             schema.CreateIndex(idx),
             "CREATE FULLTEXT INDEX test_idx1 "
             "ON testtbl (data(10)) WITH PARSER ngram",
+            dialect=dialect_name,
         )
 
-    def test_create_index_with_length(self):
+    @testing.combinations("mysql", "mariadb", argnames="dialect_name")
+    def test_create_index_with_length(self, dialect_name):
         m = MetaData()
         tbl = Table("testtbl", m, Column("data", String(255)))
-        idx1 = Index("test_idx1", tbl.c.data, mysql_length=10)
-        idx2 = Index("test_idx2", tbl.c.data, mysql_length=5)
+        idx1 = Index("test_idx1", tbl.c.data, **{f"{dialect_name}_length": 10})
+        idx2 = Index("test_idx2", tbl.c.data, **{f"{dialect_name}_length": 5})
 
         self.assert_compile(
             schema.CreateIndex(idx1),
             "CREATE INDEX test_idx1 ON testtbl (data(10))",
+            dialect=dialect_name,
         )
         self.assert_compile(
             schema.CreateIndex(idx2),
             "CREATE INDEX test_idx2 ON testtbl (data(5))",
+            dialect=dialect_name,
         )
 
     def test_drop_constraint_mysql(self):
@@ -361,19 +409,26 @@ class CompileTest(ReservedWordFixture, fixtures.TestBase, AssertsCompiledSQL):
             "CREATE INDEX test_idx3 ON testtbl (a(30), b(30))",
         )
 
-    def test_create_index_with_using(self):
+    @testing.combinations("mysql", "mariadb", argnames="dialect_name")
+    def test_create_index_with_using(self, dialect_name):
         m = MetaData()
         tbl = Table("testtbl", m, Column("data", String(255)))
-        idx1 = Index("test_idx1", tbl.c.data, mysql_using="btree")
-        idx2 = Index("test_idx2", tbl.c.data, mysql_using="hash")
+        idx1 = Index(
+            "test_idx1", tbl.c.data, **{f"{dialect_name}_using": "btree"}
+        )
+        idx2 = Index(
+            "test_idx2", tbl.c.data, **{f"{dialect_name}_using": "hash"}
+        )
 
         self.assert_compile(
             schema.CreateIndex(idx1),
             "CREATE INDEX test_idx1 ON testtbl (data) USING btree",
+            dialect=dialect_name,
         )
         self.assert_compile(
             schema.CreateIndex(idx2),
             "CREATE INDEX test_idx2 ON testtbl (data) USING hash",
+            dialect=dialect_name,
         )
 
     def test_create_pk_plain(self):
@@ -391,7 +446,67 @@ class CompileTest(ReservedWordFixture, fixtures.TestBase, AssertsCompiledSQL):
             "PRIMARY KEY (data))",
         )
 
-    def test_create_pk_with_using(self):
+    @testing.combinations("mysql", "mariadb", argnames="dialect_name")
+    def test_create_pk_with_using(self, dialect_name):
+        m = MetaData()
+        tbl = Table(
+            "testtbl",
+            m,
+            Column("data", String(255)),
+            PrimaryKeyConstraint("data", **{f"{dialect_name}_using": "btree"}),
+        )
+
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE testtbl (data VARCHAR(255) NOT NULL, "
+            "PRIMARY KEY (data) USING btree)",
+            dialect=dialect_name,
+        )
+
+    @testing.combinations(
+        ("with_parser", "ngram", "WITH PARSER ngram"),
+        ("using", "btree", "USING btree"),
+        argnames="paramname, value, expected",
+    )
+    @testing.variation("use_deprecated", [True, False])
+    def test_create_index_mysql_option_mariadb_deprecated(
+        self, paramname, value, expected, use_deprecated
+    ):
+        """Test that mysql_with_parser emits deprecation with mariadb
+        dialect"""
+        m = MetaData()
+        tbl = Table("testtbl", m, Column("data", String(255)))
+        idx = Index(
+            "test_idx1",
+            tbl.c.data,
+            mariadb_length=10,
+            mariadb_prefix="FULLTEXT",
+            **{
+                f"{'mysql' if use_deprecated else 'mariadb'}"
+                f"_{paramname}": value
+            },
+        )
+
+        if use_deprecated:
+            expect = expect_deprecated(
+                f"Using 'mysql_{paramname}' with the 'mariadb' dialect is "
+                f"deprecated; please additionally specify "
+                f"'mariadb_{paramname}'.",
+            )
+        else:
+            expect = contextlib.nullcontext()
+
+        with expect:
+            self.assert_compile(
+                schema.CreateIndex(idx),
+                "CREATE FULLTEXT INDEX test_idx1 "
+                f"ON testtbl (data(10)) {expected}",
+                dialect="mariadb",
+            )
+
+    def test_create_pk_with_using_mysql_option_mariadb_deprecated(self):
+        """Test that mysql_using for PK emits deprecation with mariadb
+        dialect"""
         m = MetaData()
         tbl = Table(
             "testtbl",
@@ -400,12 +515,116 @@ class CompileTest(ReservedWordFixture, fixtures.TestBase, AssertsCompiledSQL):
             PrimaryKeyConstraint("data", mysql_using="btree"),
         )
 
-        self.assert_compile(
-            schema.CreateTable(tbl),
-            "CREATE TABLE testtbl (data VARCHAR(255) NOT NULL, "
-            "PRIMARY KEY (data) USING btree)",
+        with expect_deprecated(
+            "Using 'mysql_using' with the 'mariadb' dialect is deprecated; "
+            "please additionally specify 'mariadb_using'."
+        ):
+            self.assert_compile(
+                schema.CreateTable(tbl),
+                "CREATE TABLE testtbl (data VARCHAR(255) NOT NULL, "
+                "PRIMARY KEY (data) USING btree)",
+                dialect="mariadb",
+            )
+
+    @testing.combinations(
+        (True, True, (10, 2, 2)),
+        (True, True, (10, 2, 1)),
+        (False, True, (10, 2, 0)),
+        (True, False, (8, 0, 14)),
+        (True, False, (8, 0, 13)),
+        (False, False, (8, 0, 12)),
+        argnames="has_brackets,is_mariadb,version",
+    )
+    def test_create_server_default_with_function_using(
+        self, has_brackets, is_mariadb, version
+    ):
+        dialect = mysql.dialect(is_mariadb=is_mariadb)
+        dialect.server_version_info = version
+
+        m = MetaData()
+        tbl = Table(
+            "testtbl",
+            m,
+            Column("time", DateTime, server_default=func.current_timestamp()),
+            Column("name", String(255), server_default="some str"),
+            Column(
+                "description", String(255), server_default=func.lower("hi")
+            ),
+            Column("data", JSON, server_default=func.json_object()),
+            Column(
+                "updated1",
+                DateTime,
+                server_default=text("now() on update now()"),
+            ),
+            Column(
+                "updated2",
+                DateTime,
+                server_default=text("now() On  UpDate now()"),
+            ),
+            Column(
+                "updated3",
+                DateTime,
+                server_default=text("now() ON UPDATE now()"),
+            ),
+            Column(
+                "updated4",
+                DateTime,
+                server_default=text("now(3)"),
+            ),
+            Column(
+                "updated5",
+                DateTime,
+                server_default=text("nOW(3)"),
+            ),
+            Column(
+                "updated6",
+                DateTime,
+                server_default=text("notnow(1)"),
+            ),
+            Column(
+                "updated7",
+                DateTime,
+                server_default=text("CURRENT_TIMESTAMP(3)"),
+            ),
         )
 
+        eq_(dialect._support_default_function, has_brackets)
+
+        if has_brackets:
+            self.assert_compile(
+                schema.CreateTable(tbl),
+                "CREATE TABLE testtbl ("
+                "time DATETIME DEFAULT CURRENT_TIMESTAMP, "
+                "name VARCHAR(255) DEFAULT 'some str', "
+                "description VARCHAR(255) DEFAULT (lower('hi')), "
+                "data JSON DEFAULT (json_object()), "
+                "updated1 DATETIME DEFAULT now() on update now(), "
+                "updated2 DATETIME DEFAULT now() On  UpDate now(), "
+                "updated3 DATETIME DEFAULT now() ON UPDATE now(), "
+                "updated4 DATETIME DEFAULT now(3), "
+                "updated5 DATETIME DEFAULT nOW(3), "
+                "updated6 DATETIME DEFAULT (notnow(1)), "
+                "updated7 DATETIME DEFAULT CURRENT_TIMESTAMP(3))",
+                dialect=dialect,
+            )
+        else:
+            self.assert_compile(
+                schema.CreateTable(tbl),
+                "CREATE TABLE testtbl ("
+                "time DATETIME DEFAULT CURRENT_TIMESTAMP, "
+                "name VARCHAR(255) DEFAULT 'some str', "
+                "description VARCHAR(255) DEFAULT lower('hi'), "
+                "data JSON DEFAULT json_object(), "
+                "updated1 DATETIME DEFAULT now() on update now(), "
+                "updated2 DATETIME DEFAULT now() On  UpDate now(), "
+                "updated3 DATETIME DEFAULT now() ON UPDATE now(), "
+                "updated4 DATETIME DEFAULT now(3), "
+                "updated5 DATETIME DEFAULT nOW(3), "
+                "updated6 DATETIME DEFAULT notnow(1), "
+                "updated7 DATETIME DEFAULT CURRENT_TIMESTAMP(3))",
+                dialect=dialect,
+            )
+
     def test_create_index_expr(self):
         m = MetaData()
         t1 = Table("foo", m, Column("x", Integer))
@@ -673,6 +892,14 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             .with_dialect_options(mysql_limit=5),
             "UPDATE t SET col1=%s LIMIT 5",
         )
+
+        # does not make sense but we want this to compile
+        self.assert_compile(
+            t.update()
+            .values({"col1": 123})
+            .with_dialect_options(mysql_limit=0),
+            "UPDATE t SET col1=%s LIMIT 0",
+        )
         self.assert_compile(
             t.update()
             .values({"col1": 123})
@@ -687,6 +914,39 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             "UPDATE t SET col1=%s WHERE t.col2 = %s LIMIT 1",
         )
 
+    def test_delete_limit(self):
+        t = sql.table("t", sql.column("col1"), sql.column("col2"))
+
+        self.assert_compile(t.delete(), "DELETE FROM t")
+        self.assert_compile(
+            t.delete().with_dialect_options(mysql_limit=5),
+            "DELETE FROM t LIMIT 5",
+        )
+        # does not make sense but we want this to compile
+        self.assert_compile(
+            t.delete().with_dialect_options(mysql_limit=0),
+            "DELETE FROM t LIMIT 0",
+        )
+        self.assert_compile(
+            t.delete().with_dialect_options(mysql_limit=None),
+            "DELETE FROM t",
+        )
+        self.assert_compile(
+            t.delete()
+            .where(t.c.col2 == 456)
+            .with_dialect_options(mysql_limit=1),
+            "DELETE FROM t WHERE t.col2 = %s LIMIT 1",
+        )
+
+    @testing.combinations((update,), (delete,))
+    def test_update_delete_limit_int_only(self, crud_fn):
+        t = sql.table("t", sql.column("col1"), sql.column("col2"))
+
+        with expect_raises(ValueError):
+            crud_fn(t).with_dialect_options(mysql_limit="not an int").compile(
+                dialect=mysql.dialect()
+            )
+
     def test_utc_timestamp(self):
         self.assert_compile(func.utc_timestamp(), "utc_timestamp()")
 
@@ -931,6 +1191,36 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             ")ENGINE=InnoDB",
         )
 
+    @testing.combinations(
+        (Sequence("foo_seq"), "CREATE SEQUENCE foo_seq"),
+        (Sequence("foo_seq", cycle=True), "CREATE SEQUENCE foo_seq CYCLE"),
+        (Sequence("foo_seq", cycle=False), "CREATE SEQUENCE foo_seq NOCYCLE"),
+        (
+            Sequence(
+                "foo_seq",
+                start=1,
+                increment=2,
+                nominvalue=True,
+                nomaxvalue=True,
+                cycle=False,
+                cache=100,
+            ),
+            (
+                "CREATE SEQUENCE foo_seq INCREMENT BY 2 START WITH 1 NO"
+                " MINVALUE NO MAXVALUE CACHE 100 NOCYCLE"
+            ),
+        ),
+        argnames="seq, expected",
+    )
+    @testing.variation("use_mariadb", [True, False])
+    def test_mariadb_sequence_behaviors(self, seq, expected, use_mariadb):
+        """test #13073"""
+        self.assert_compile(
+            CreateSequence(seq),
+            expected,
+            dialect="mariadb" if use_mariadb else "mysql",
+        )
+
     def test_create_table_with_partition(self):
         t1 = Table(
             "testtable",
@@ -1249,6 +1539,25 @@ class InsertOnDuplicateTest(fixtures.TestBase, AssertsCompiledSQL):
             dialect=dialect,
         )
 
+    def test_on_update_instrumented_attribute_dict(self):
+        class Base(DeclarativeBase):
+            pass
+
+        class T(Base):
+            __tablename__ = "table"
+
+            foo: Mapped[int] = mapped_column(Integer, primary_key=True)
+
+        q = insert(T).values(foo=1).on_duplicate_key_update({T.foo: 2})
+        self.assert_compile(
+            q,
+            (
+                "INSERT INTO `table` (foo) VALUES (%s) "
+                "ON DUPLICATE KEY UPDATE foo = %s"
+            ),
+            {"foo": 1, "param_1": 2},
+        )
+
 
 class RegexpCommon(testing.AssertsCompiledSQL):
     def setup_test(self):
