@@ -31,6 +31,7 @@ from sqlalchemy.testing import is_not
 from sqlalchemy.testing import is_not_none
 from sqlalchemy.testing import is_true
 from sqlalchemy.testing import mock
+from sqlalchemy.testing.assertions import expect_raises_message
 from sqlalchemy.testing.engines import testing_engine
 from sqlalchemy.testing.util import gc_collect
 from sqlalchemy.testing.util import lazy_gc
@@ -164,6 +165,25 @@ class PoolTest(PoolTestBase):
         assert not c2.info
         assert "foo2" in c.info
 
+    def test_raw_connection_context(self):
+        p = self._queuepool_fixture(pool_size=1, max_overflow=0)
+        e = create_engine("sqlite://", pool=p, _initialize=False)
+
+        eq_(p.checkedout(), 0)
+        conn = e.raw_connection()
+        eq_(p.checkedout(), 1)
+        with conn as wc:
+            eq_(p.checkedout(), 1)
+            is_(conn, wc)
+
+        eq_(p.checkedout(), 0)
+
+        with expect_raises_message(ValueError, "an error"):
+            with e.raw_connection() as wc:
+                eq_(p.checkedout(), 1)
+                raise ValueError("an error")
+        eq_(p.checkedout(), 0)
+
     def test_rec_info(self):
         p = self._queuepool_fixture(pool_size=1, max_overflow=0)
 
@@ -1083,16 +1103,32 @@ class QueuePoolTest(PoolTestBase):
         assert_raises(tsa.exc.TimeoutError, p.connect)
         assert int(time.time() - now) == 2
 
-    @testing.requires.timing_intensive
     def test_timeout_subsecond_precision(self):
-        p = self._queuepool_fixture(pool_size=1, max_overflow=0, timeout=0.5)
-        c1 = p.connect()  # noqa
-        with expect_raises(tsa.exc.TimeoutError):
-            now = time.time()
-            c2 = p.connect()  # noqa
-        # Python timing is not very accurate, the time diff should be very
-        # close to 0.5s but we give 200ms of slack.
-        assert 0.3 <= time.time() - now <= 0.7, "Pool timeout not respected"
+        """test that a subsecond precision is passed to queue.get()
+        and that the math comparison is float sensitive
+
+        """
+        times = [
+            # will set endtime at 1761659753.1250672 + .5
+            1761659753.1250672,
+            # will be within the timeout
+            1761659753.349,
+            # will be outside the timeout, should raise Empty
+            # if the timeout is a whole number, then this would not be
+            # enough time to wait and we get "pop from an empty list"
+            1761659753.715,
+        ]
+
+        def mock_time():
+            return times.pop(0)
+
+        with mock.patch("sqlalchemy.util.queue._time", mock_time):
+            p = self._queuepool_fixture(
+                pool_size=1, max_overflow=0, timeout=0.5
+            )
+            c1 = p.connect()  # noqa
+            with expect_raises(tsa.exc.TimeoutError):
+                p.connect()
 
     @testing.requires.threading_with_mock
     @testing.requires.timing_intensive
